#!/usr/bin/env bash
set -e

TARGET_NAME=$2
MAC_ADDRESS=$4
MAC_ADDRESS_ETH1=$6

DEVICE=/dev/mmcblk0

FAT_PARTITION_1=/dev/mmcblk0p1
FAT_PARTITION_2=/dev/mmcblk0p2

EXT_PARTITION_1=/dev/mmcblk0p5
EXT_PARTITION_2=/dev/mmcblk0p6
EXT_MOUNTING_POINT_1="/tmp/ext_mounting_point_1"
EXT_MOUNTING_POINT_2="/tmp/ext_mounting_point_2"

UENV_TXT="/tmp/u-boot-env.txt"

function erase_partition_table
{
    dd if=/dev/zero of=${DEVICE} bs=1M count=1
}


function create_partitions
{
    if [ "$TARGET_NAME" == "sketchboard" ]; then
        sector_number=59008
    else
        sector_number=119040
    fi

    echo "Creating partitions:"
    sed -e 's/\([\+0-9a-zA-Z]*\).*/\1/' << EOF | fdisk ${DEVICE}
o # clear partition table
n # new partition (for FAT32):
p #     primary
1 #     partition number
  #     default, start from beginning of disk
+20M # size
n # new partition (for FAT32):
p #     primary
2 #     partition number
  #     default, start from beginning of disk
+20M # size
n # new partition (for a2):
p #     primary
3 #     partition number
  #     default, start after preceding partition
+1M #   size
n # new partition :
e #     extended
  #     default, start after preceding partition
  # max size size
n # new partition (for Linux)
  # default start
$sector_number # half of available space
n # new partition (for Linux)
  # default start
  # till end of disk
a # make bootable partition:
1 #     partition number
t # change partition system id:
1 #     partition number
b #     FAT32
t # change partition system id:
2 #     partition number
b #     FAT32
t # change partition system id:
3 #     partition number
a2 #    Unknown
t # change partition system id:
5 #     partition number
83 #    Linux
t # change partition system id:
6 #     partition number
83 #    Linux
p # print partition table
w # write/quit
EOF
}


function format_partitions
{
    echo "Formatting partitions:"
    echo "   Formatting ${FAT_PARTITION_1}"
    mkfs.vfat -n BOOT1 ${FAT_PARTITION_1}
    echo "   Formatting ${FAT_PARTITION_2}"
    mkfs.vfat -n BOOT2 ${FAT_PARTITION_2}
    echo "   Formatting ${EXT_PARTITION_1}"
    mkfs.ext4 -F ${EXT_PARTITION_1} -L rootfs5
    echo "   Formatting ${EXT_PARTITION_2}"
    mkfs.ext4 -F ${EXT_PARTITION_2} -L rootfs6
}


function mount_partition
{
    local DEVICE=$1
    local MOUNT_POINT=$2
    echo "Mounting ${DEVICE} to ${MOUNT_POINT}"

    mkdir -p ${MOUNT_POINT}
    mount ${DEVICE} ${MOUNT_POINT}
}


function unmount_partitions
{
    local DEVICE=$1

    for n in ${DEVICE}* ; do umount $n 2> /dev/null || /bin/true ; done
}

function create_environments
{
    sed -e "s/ethaddr=.*/ethaddr=${MAC_ADDRESS}/g" \
        -e "s/eth1addr=.*/eth1addr=${MAC_ADDRESS_ETH1}/g" \
        -i ${1}
}

function write_uboot_enviroments
{
    local device=$1
    local env_txt=$2
    local uboot_env_image="/tmp/u-boot-env.bin"

    mkenvimage -s 4096 -o ${uboot_env_image} ${env_txt}
    dd if=${uboot_env_image} of=${device} bs=512 count=8  seek=1 conv=fsync
}

FIRMWARE_PATH=/tmp/firmware

rm -fr ${FIRMWARE_PATH}
mkdir -p ${FIRMWARE_PATH}
tar xf /tmp/firmware.tar.gz -C ${FIRMWARE_PATH}

echo "Erasing partition table:"
erase_partition_table
create_partitions
format_partitions

echo "Writing u-boot:"
dd if=${FIRMWARE_PATH}/u-boot-with-spl.sfp of=${DEVICE}p3 bs=64k

echo "Mounting ext partitions:"
mount_partition ${EXT_PARTITION_1} ${EXT_MOUNTING_POINT_1}
mount_partition ${EXT_PARTITION_2} ${EXT_MOUNTING_POINT_2}

echo "Untarring rootfs:"
tar -xf ${FIRMWARE_PATH}/rootfs.tar -C ${EXT_MOUNTING_POINT_1} > /dev/null 2>&1
tar -xf ${FIRMWARE_PATH}/rootfs.tar -C ${EXT_MOUNTING_POINT_2} > /dev/null 2>&1

echo "Creating u-boot environment variables:"
create_environments ${UENV_TXT}

echo "Writing u-boot environment image:"
write_uboot_enviroments ${DEVICE} ${UENV_TXT}
sync

echo "Unmount partitions:"
unmount_partitions ${DEVICE}
